@@ -592,15 +592,20 @@ founddef(char *defstr, int blkid, int expctr, int hasval, int elen)
 
 	if (!SLIST_EMPTY(&varhead)) {
 		SLIST_FOREACH_SAFE(v1, &varhead, entry, vt) {
-			if (strcmp(vnamep, v1->v_name) == 0)
+			if (strcmp(vnamep, v1->v_name) == 0) {
 				SLIST_REMOVE(&varhead, v1, varentry, entry);
+				free(v1->v_name);
+				free(v1);
+			}
 		}
 	}
 	if ((v1 = malloc(sizeof(struct varentry))) == NULL)
 		return (ABORT);
-	SLIST_INSERT_HEAD(&varhead, v1, entry);
-	if ((v1->v_name = strndup(vnamep, BUFSIZE)) == NULL)
+	if ((v1->v_name = strndup(vnamep, BUFSIZE)) == NULL) {
+		free(v1);
 		return(dobeep_msg("strndup error"));
+	}
+	SLIST_INSERT_HEAD(&varhead, v1, entry);
 	vnamep = v1->v_name;
 	v1->v_count = 0;
 	v1->v_vals = NULL;
